@@ -90,6 +90,12 @@ impl Renderer for ItemRenderer {
                         .context(format!("Could not read file with path {:?}", &dataset.path))?;
                     let headers = reader.headers()?.iter().map(|s| s.to_owned()).collect_vec();
 
+                    let table_specs = &table_specs
+                        .clone()
+                        .into_iter()
+                        .filter(|(k, s)| !s.optional || headers.contains(k))
+                        .collect();
+
                     let additional_headers = if dataset.header_rows > 1 {
                         let mut additional_header_reader = generate_reader().context(format!(
                             "Could not read file with path {:?}",
@@ -124,7 +130,6 @@ impl Renderer for ItemRenderer {
                                 .collect_vec(),
                             &headers,
                             &self.specs.views.keys().map(|s| s.to_owned()).collect_vec(),
-                            table_specs,
                             name,
                             table.description.as_deref(),
                             &linked_tables,
@@ -178,7 +183,6 @@ fn render_page<P: AsRef<Path>>(
     data: Vec<&StringRecord>,
     titles: &[String],
     tables: &[String],
-    render_columns: &HashMap<String, RenderColumnSpec>,
     name: &str,
     description: Option<&str>,
     linked_tables: &LinkedTable,
@@ -206,11 +210,7 @@ fn render_page<P: AsRef<Path>>(
     } else {
         None
     };
-    let data = data
-        .iter()
-        .enumerate()
-        .map(|(i, r)| link_columns(render_columns, titles, r.to_vec(), i).unwrap())
-        .collect_vec();
+
     let compressed_data = compress_to_utf16(&json!(data).to_string());
 
     let local: DateTime<Local> = Local::now();
@@ -274,7 +274,11 @@ fn render_table_javascript<P: AsRef<Path>>(
     let custom_plots: HashMap<String, CustomPlot> = render_columns
         .iter()
         .filter(|(_, k)| k.custom_plot.is_some())
-        .map(|(k, v)| (k.to_owned(), v.custom_plot.as_ref().unwrap().to_owned()))
+        .map(|(k, v)| {
+            let mut custom_plot = v.custom_plot.as_ref().unwrap().to_owned();
+            custom_plot.read_schema().unwrap();
+            (k.to_owned(), custom_plot)
+        })
         .collect();
 
     let tick_plots: HashMap<String, String> = render_columns
@@ -318,6 +322,30 @@ fn render_table_javascript<P: AsRef<Path>>(
         })
         .collect();
 
+    let link_urls: HashMap<String, String> = render_columns
+        .iter()
+        .filter(|(_, k)| k.link_to_url.is_some())
+        .map(|(t, spec)| {
+            (
+                t.to_string(),
+                spec.link_to_url.as_ref().unwrap().to_string(),
+            )
+        })
+        .collect();
+
+    let mut display_modes: HashMap<String, String> = titles
+        .iter()
+        .map(|t| (t.to_string(), "normal".to_string()))
+        .collect();
+    for (title, rc) in render_columns {
+        display_modes.insert(title.to_string(), rc.display_mode.to_string());
+    }
+    let detail_mode = display_modes
+        .iter()
+        .filter(|(_, mode)| *mode != "normal")
+        .count()
+        > 0;
+
     let header_rows = additional_headers.map(|headers| {
         headers
             .iter()
@@ -336,6 +364,9 @@ fn render_table_javascript<P: AsRef<Path>>(
     context.insert("custom_plots", &custom_plots);
     context.insert("tick_plots", &tick_plots);
     context.insert("heatmaps", &heatmaps);
+    context.insert("display_modes", &display_modes);
+    context.insert("detail_mode", &detail_mode);
+    context.insert("link_urls", &link_urls);
     context.insert("num", &numeric);
     context.insert("pin_columns", &pin_columns);
 
@@ -349,37 +380,6 @@ fn render_table_javascript<P: AsRef<Path>>(
     Ok(())
 }
 
-/// Apply render columns specs to a single table row
-fn link_columns(
-    render_columns: &HashMap<String, RenderColumnSpec>,
-    titles: &[String],
-    column: Vec<String>,
-    row: usize,
-) -> Result<Vec<String>> {
-    let mut result = Vec::new();
-    for (i, title) in titles.iter().enumerate() {
-        if let Some(render_column) = render_columns.get(title) {
-            if let Some(link) = render_column.link_to_url.clone() {
-                result.push(format!(
-                    "<a href='{}' target='_blank' >{}</a>",
-                    link.replace("{value}", &column[i]),
-                    column[i]
-                ));
-            } else if render_column.custom_plot.is_some() || render_column.plot.is_some() {
-                result.push(format!(
-                    "<div id='{}-{}' data-value='{}'>{}</div>",
-                    title, row, column[i], column[i]
-                ));
-            } else {
-                result.push(column[i].to_string());
-            }
-        } else {
-            result.push(column[i].to_string());
-        }
-    }
-    Ok(result)
-}
-
 /// Renders an empty page when datasets are empty
 fn render_empty_dataset<P: AsRef<Path>>(
     output_path: P,
@@ -516,7 +516,11 @@ fn render_tick_plot(
         .headers()
         .map(|s| s.iter().position(|t| t == title).unwrap())?;
 
-    let (min, max) = get_min_max(csv_path, separator, column_index, header_rows)?;
+    let (min, max) = if let Some(domain) = &tick_plot.domain {
+        (domain[0], domain[1])
+    } else {
+        get_min_max(csv_path, separator, column_index, header_rows)?
+    };
 
     let mut templates = Tera::default();
     templates.add_raw_template(
@@ -539,6 +543,10 @@ fn get_column_domain(
     header_rows: usize,
     heatmap: &Heatmap,
 ) -> Result<String> {
+    if let Some(domain) = &heatmap.domain {
+        return Ok(json!(domain).to_string());
+    }
+
     let mut reader = csv::ReaderBuilder::new()
         .delimiter(separator as u8)
         .from_path(csv_path)
@@ -616,6 +624,9 @@ fn render_plot_page<P: AsRef<Path>>(
         }
     }
 
+    let mut render_plot_specs = item_spec.render_plot.clone().unwrap();
+    render_plot_specs.read_schema()?;
+
     let mut templates = Tera::default();
     templates.add_raw_template(
         "plot.html.tera",
@@ -629,7 +640,7 @@ fn render_plot_page<P: AsRef<Path>>(
     context.insert("description", &item_spec.description);
     context.insert("tables", tables);
     context.insert("name", name);
-    context.insert("specs", &item_spec.render_plot.as_ref().unwrap().schema);
+    context.insert("specs", &render_plot_specs.schema.as_ref().unwrap());
     context.insert("time", &local.format("%a %b %e %T %Y").to_string());
     context.insert("version", &env!("CARGO_PKG_VERSION"));
 
@@ -680,7 +691,7 @@ fn render_linkouts(
     let mut linkouts = Vec::new();
     for (name, link_specs) in links {
         let index = titles.iter().position(|t| t == &link_specs.column).unwrap();
-        if let Some(table) = &link_specs.item {
+        if let Some(table) = &link_specs.view {
             let val = table.replace("{value}", &row[index]);
             linkouts.push(Linkout {
                 name: name.to_string(),
